
from mdai.visualize import load_dicom_image
from keras.utils import Sequence, to_categorical

# from skimage.transform import resize
import numpy as np
from PIL import Image


class DataGenerator(Sequence):
    def __init__(
        self, dataset, batch_size=32, dim=(32, 32), n_channels=1, n_classes=10, shuffle=True
    ):
        """Generates data for Keras fit_generator() function.
        """

        # Initialization
        self.dim = dim
        self.batch_size = batch_size

        self.img_ids = dataset.image_ids
        self.imgs_anns = dataset.imgs_anns
        self.dataset = dataset

        self.n_channels = n_channels
        self.n_classes = n_classes
        self.shuffle = shuffle
        self.on_epoch_end()

    def __len__(self):
        "Denotes the number of batches per epoch"
        return int(np.floor(len(self.img_ids) / self.batch_size))

    def __getitem__(self, index):
        "Generate one batch of data"

        # Generate indexes of the batch
        indexes = self.indexes[index * self.batch_size : (index + 1) * self.batch_size]

        # Find list of IDs
        img_ids_temp = [self.img_ids[k] for k in indexes]

        # Generate data
        X, y = self.__data_generation(img_ids_temp)

        return X, y

    def on_epoch_end(self):
        "Updates indexes after each epoch"
        self.indexes = np.arange(len(self.img_ids))
        if self.shuffle:
            np.random.shuffle(self.indexes)

    def __data_generation(self, img_ids_temp):
        "Generates data containing batch_size samples"

        # Initialization
        X = np.empty((self.batch_size, *self.dim, self.n_channels))
        y = np.empty((self.batch_size), dtype=int)

        # Generate data
        for i, ID in enumerate(img_ids_temp):
            image = load_dicom_image(ID, to_RGB=True)
            image = Image.fromarray(image)
            image = image.resize((self.dim[0], self.dim[1]))
            # image = resize(image, (self.dim[0], self.dim[1]))

            X[i,] = image

            ann = self.imgs_anns[ID][0]
            y[i] = self.dataset.labels_dict[ann["labelId"]]["class_id"]
        return X, to_categorical(y, num_classes=self.n_classes)
